/*
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.sun.s1asdev.ejb.ejb30.hello.session2;

import javax.ejb.Stateful;
import javax.ejb.Remote;
import javax.ejb.EJB;
import javax.annotation.PostConstruct;
import javax.interceptor.Interceptors;
import javax.ejb.EJBs;
import javax.ejb.Remove;
import javax.annotation.PreDestroy;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.naming.InitialContext;

import javax.annotation.Resource;
import javax.transaction.UserTransaction;

import java.util.Collection;
import java.util.HashSet;
import java.io.*;

@Stateful
@TransactionManagement(TransactionManagementType.BEAN)

@EJBs( 
 { @EJB(name="ejb/TypeLevelSless1", beanName="SlessEJB", 
       beanInterface=Sless.class),
   @EJB(name="ejb/TypeLevelSless2", beanName="SlessEJB2", 
       beanInterface=Sless.class) ,
   @EJB(name="ejb/SfulEJB2", beanInterface=Sful2.class)
 })

@Interceptors(MyCallbackHandler.class)
@EJB(name="ejb/TypeLevelSless3", beanInterface=SlessSub.class)
@Remote({Sful.class})
// ejb is not required to put business interface in implements clause
public class SfulEJB 
{

    private @Resource SessionContext sc;

    private @EJB(beanName="SlessEJB") Sless sless;
    private @EJB(beanName="SlessEJB2") Sless sless2;
    private @EJB(beanName="SlessEJB3") SlessSub sless3;


    @Resource
    private void setSessionContext(SessionContext context) {

        System.out.println("********Sful: setSessionContext with @Resource");

    }

    private String state;

    public void set(String value) {
        state = value;
    }

    public String get() {
        return state;
    }

    public String hello() {
        System.out.println("In SfulEJB:hello()");

        try {

            testSlessRefs(sless, sless2, sless3);

        } catch(Exception e) {
            e.printStackTrace();
            throw new EJBException("hello failure", e);
        }

        return "hello";
    }
  /*
    public void foo() {}
    public void foo(int a, String b) {
    }
    */
    private void testSlessRefs(Sless s1, Sless s2, SlessSub s3) 
        throws Exception {



        System.out.println("************testSlessRefs*********** ");
        String sless1Id = s1.getId();
        System.out.println("sless1Id = " + sless1Id);
        String sless2Id = s2.getId();
        System.out.println("sless2Id = " + sless2Id);
        String sless3Id = s3.getId();
        System.out.println("sless3Id = " + sless3Id);

    }

    @PostConstruct
    public void afterCreate() {
        System.out.println("*******In SfulEJB::afterCreate() marked as PostConstruct");
        /*
        // jndi lookup should work within postconstruct
        Sless sless = (Sless) sc.lookup("ejb/TypeLevelSless1");

        // Access to getBusinessObject is allowed here
        sc.getBusinessObject(Sful.class);
        */
    }
    

    @PreDestroy
    public void myPreDestroy() {
        try {
            System.out.println("**********SfulEJB:myPreDestroy by @PreDestroy **************");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Remove
    public void remove() {
        System.out.println("In SfulEJB " + state + " @Remove method");
        // Access to getBusinessObject is allowed here
        sc.getBusinessObject(Sful.class);

    }

    @Remove(retainIfException=true)
    public void removeRetainIfException(boolean throwException) 
        throws Exception {

        System.out.println("In SfulEJB " + state + " removeRetainIfException");
        System.out.println("throwException = " + throwException);
        if( throwException ) {
            throw new Exception("throwing app exception from @Remove method");
        }
    }

    @Remove
    public void removeNotRetainIfException(boolean throwException) 
        throws Exception {

        System.out.println("In SfulEJB " + state + 
                           "removeNotRetainIfException");
        System.out.println("throwException = " + throwException);
        if( throwException ) {
            throw new Exception("throwing app exception from @Remove method");
        }
    }

    @Remove
    public void removeMethodThrowSysException(boolean throwException) {

        System.out.println("In SfulEJB " + state + 
                           "removeMethodThrowSysException");
        System.out.println("throwException = " + throwException);
        if( throwException ) {
            throw new EJBException
                ("throwing system exception from @Remove method");
        }
    }

    public void doRemoveMethodSessionSyncTests() {

        Sful2 sful2_1 = (Sful2) sc.lookup("ejb/SfulEJB2");
        Sful2 sful2_2 = (Sful2) sc.lookup("ejb/SfulEJB2");
        Sful2 sful2_3 = (Sful2) sc.lookup("ejb/SfulEJB2");

        sful2_1.hello();
        try {
            sful2_1.removeRetainIfException(true);
        } catch(Exception e) {
            e.printStackTrace();
        }

        // all SessionSynch callbacks should have happened
        if( SfulEJB2.afterBeginCalled &&
            SfulEJB2.beforeCompletionCalled &&
            SfulEJB2.afterCompletionCalled ) {
            System.out.println("Got expected SessionSynch behavior for " +
                               "removeRetainIfException(true)");                               
        } else {
            throw new EJBException("SessionSynch failure for " +
                                   "removeRetainIfException(true)");
        }

        // invocation should still work since bean wasn't removed
        sful2_1.hello();

        try {
            sful2_1.removeRetainIfException(false);
        } catch(Exception e) {
            throw new EJBException(e);
        }

        // bean should have been removed and completion-time SessionSynch
        // callbacks should not have been called.
        if( SfulEJB2.afterBeginCalled &&
            !SfulEJB2.beforeCompletionCalled &&
            !SfulEJB2.afterCompletionCalled ) {
            System.out.println("Got expected SessionSynch behavior for " +
                               "removeRetainIfException(false)");         
        } else {
            throw new EJBException("SessionSynch failure for " +
                                   "removeRetainIfException(true)");
        }

        



    }


}
